"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var jsx_runtime_1 = require("@emotion/react/jsx-runtime");
var react_1 = require("@emotion/react");
var react_2 = require("react");
var useDebounce_1 = require("../../../hooks/useDebounce");
var useThrottle_1 = require("../../../hooks/useThrottle");
var props_1 = require("../../../utilities/props");
var Styled = require("./style");
var css_1 = require("../../../utilities/css");
var VARIANTS = {
    text: Styled.TextButton,
    contained: Styled.ContainedButton,
    outlined: Styled.OutlinedButton,
};
/**
 * Button Component
 */
var Button = (0, react_2.forwardRef)(function (props, forwardedRef) {
    var theme = (0, react_1.useTheme)();
    var _a = props.onClick, onClick = _a === void 0 ? function () { } : _a, _b = props.variant, variant = _b === void 0 ? "contained" : _b, _c = props.debounce, debounce = _c === void 0 ? 0 : _c, _d = props.throttle, throttle = _d === void 0 ? 0 : _d;
    var throttledFunction = (0, useThrottle_1.default)(onClick, throttle);
    var debouncedFunction = (0, useDebounce_1.default)(throttledFunction, debounce);
    var handleClick = (0, react_2.useCallback)(function (event) {
        rippleEffect(event, theme, variant);
        debouncedFunction(event);
    }, [onClick]);
    var style = __assign(__assign({}, (0, props_1.getElementProps)(props)), props.style);
    var Button = VARIANTS[variant];
    return ((0, jsx_runtime_1.jsx)(Button, __assign({}, props, { ref: forwardedRef, onClick: handleClick, style: style })));
});
exports.default = Button;
/**
 * Generate Ripple Effect at Button Click
 * @param event
 * @param theme
 * @param variant
 */
var rippleEffect = function (event, theme, variant) {
    var button = event.currentTarget;
    var container = document.createElement("div");
    container.style.width = "".concat(button.offsetWidth, "px");
    container.style.height = "".concat(button.offsetHeight, "px");
    container.style.borderRadius = "inherit";
    container.classList.add("container");
    var circle = document.createElement("span");
    var diameter = Math.max(button.clientWidth, button.clientHeight);
    var radius = diameter / 2;
    circle.style.width = circle.style.height = "".concat(diameter, "px");
    circle.style.top = "".concat(event.pageY - button.offsetTop - radius, "px");
    circle.style.left = "".concat(event.pageX - button.offsetLeft - radius, "px");
    circle.style.backgroundColor =
        variant === "contained" ? "#fff" : (0, css_1.getPrimaryColor)(theme);
    circle.style.opacity = "1";
    circle.classList.add("ripple");
    var ripple = button.getElementsByClassName("container")[0];
    if (ripple) {
        ripple.remove();
    }
    container.appendChild(circle);
    button.appendChild(container);
};
