var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { jsx as _jsx } from "@emotion/react/jsx-runtime";
import { useTheme } from "@emotion/react";
import { forwardRef, useCallback } from "react";
import uesDebounce from "../../../hooks/useDebounce";
import useThrottle from "../../../hooks/useThrottle";
import { getElementProps } from "../../../utilities/props";
import * as Styled from "./style";
import { getPrimaryColor } from "../../../utilities/css";
var VARIANTS = {
    text: Styled.TextButton,
    contained: Styled.ContainedButton,
    outlined: Styled.OutlinedButton,
};
/**
 * Button Component
 */
var Button = forwardRef(function (props, forwardedRef) {
    var theme = useTheme();
    var _a = props.onClick, onClick = _a === void 0 ? function () { } : _a, _b = props.variant, variant = _b === void 0 ? "contained" : _b, _c = props.debounce, debounce = _c === void 0 ? 0 : _c, _d = props.throttle, throttle = _d === void 0 ? 0 : _d;
    var throttledFunction = useThrottle(onClick, throttle);
    var debouncedFunction = uesDebounce(throttledFunction, debounce);
    var handleClick = useCallback(function (event) {
        rippleEffect(event, theme, variant);
        debouncedFunction(event);
    }, [onClick]);
    var style = __assign(__assign({}, getElementProps(props)), props.style);
    var Button = VARIANTS[variant];
    return (_jsx(Button, __assign({}, props, { ref: forwardedRef, onClick: handleClick, style: style })));
});
export default Button;
/**
 * Generate Ripple Effect at Button Click
 * @param event
 * @param theme
 * @param variant
 */
var rippleEffect = function (event, theme, variant) {
    var button = event.currentTarget;
    var container = document.createElement("div");
    container.style.width = "".concat(button.offsetWidth, "px");
    container.style.height = "".concat(button.offsetHeight, "px");
    container.style.borderRadius = "inherit";
    container.classList.add("container");
    var circle = document.createElement("span");
    var diameter = Math.max(button.clientWidth, button.clientHeight);
    var radius = diameter / 2;
    circle.style.width = circle.style.height = "".concat(diameter, "px");
    circle.style.top = "".concat(event.pageY - button.offsetTop - radius, "px");
    circle.style.left = "".concat(event.pageX - button.offsetLeft - radius, "px");
    circle.style.backgroundColor =
        variant === "contained" ? "#fff" : getPrimaryColor(theme);
    circle.style.opacity = "1";
    circle.classList.add("ripple");
    var ripple = button.getElementsByClassName("container")[0];
    if (ripple) {
        ripple.remove();
    }
    container.appendChild(circle);
    button.appendChild(container);
};
